/* ------------------------------------------------------------------------- */
/* detect -- look who's there. Gets address acks from all devices on the bus.*/
/*		It should not change any values in the peripherals.	     */
/* ------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>


/* some common i2c chip addresses on bt848 grabber boards */
static struct BTTV_LIST {
    int   addr;
    char *name;
} known[] = {
    { 0x20, "saa6588 RDS/RDBS decoder (KNC1?)" },
    { 0x22, "saa5249 videotext" },
    { 0x30, "Hauppauge IR (pic16c54 onboard)" },
    { 0x34, "Hauppauge IR (pic16c54 external)" },
    { 0x80, "msp34xx/tea6300 sound processor" },
    { 0x82, "TDA8425 audio chip" },
    { 0x86, "TDA9885/TDA9886/TDA9887 tv and sound demodulator" },
    { 0x88, "Bt832" },
    { 0x8a, "TDA7432 or BT832" },
    { 0xa0, "eeprom" },
    { 0xb6, "TDA9850/TDA9873/etc. audio chip" },
    { 0xc0, "tuner" },
    { 0xc2, "tuner" },
    { 0xee, "eeprom (STB boards)" },
    { 0, NULL }
};

int
main(int argc, char *argv[])
{
    int i,j,f,c,r;
    char *device = "/dev/i2c-0";
    int addr = -1;
	
    /* parse options */
    while (-1 != (c=getopt(argc,argv,"hd:a:"))) {
	switch (c){
	case 'd':
	    device = optarg;
	    break;
	case 'a':
	    addr = strtol(optarg,NULL,0);
	    break;
	case 'h':
	default:
	    printf("This tool tries to detect devices on the i2c-bus\n");
	    printf("usage: %s [ -d device ]\n",argv[0]);
	    exit(1);
	}
    }

    if (-1 == (f = open(device,O_RDWR))) {
	fprintf(stderr,"open %s: %s\n",device,strerror(errno));
	exit(1);
    }
    for (i=0; i < 256; i += 2){
	if (-1 != addr && i != addr)
	    continue;
	fprintf(stderr,"0x%x\r",i);
	for (j = 0; known[j].name != NULL; j++)
	    if (known[j].addr == i)
		break;

	if (-1 == ioctl(f,I2C_SLAVE,i>>1)) {
	    printf("0x%x: %s (error: %s)\n",i,
		   known[j].name ? known[j].name : "???",
		   strerror(errno));
	    continue;
	}

	{
		struct i2c_smbus_ioctl_data mycmd;
		mycmd.read_write=0;
		mycmd.command=0;
		mycmd.size=I2C_SMBUS_QUICK;  // this means "probe if this address replies"
		mycmd.data=NULL;
		r=ioctl(f,I2C_SMBUS,&mycmd);
	}
	if(r==0) printf("0x%x: %s\n",i,
		   known[j].name ? known[j].name : "???");
    }
    close(f);
    exit(0);
}
